@@ -1,8 +1,10 @@
+//#include <ql/time/asx.hpp>
 #include <ql/time/businessdayconvention.hpp>
 #include <ql/time/calendar.hpp>
 #include <ql/time/date.hpp>
 #include <ql/time/dategenerationrule.hpp>
-#include <ql/time/daycounter.hpp>
+// -- not needed  #include <ql/time/daycounter.hpp>
+#include <ql/time/ecb.hpp>
 #include <ql/time/frequency.hpp>
 #include <ql/time/imm.hpp>
 #include <ql/time/period.hpp>
@@ -11,4 +13,4 @@
 #include <ql/time/weekday.hpp>
 
 #include <ql/time/calendars/all.hpp>
-#include <ql/time/daycounters/all.hpp>
+// -- not needed  #include <ql/time/daycounters/all.hpp>
